@@ -217,7 +217,8 @@ QPixmap QSvgIconEngine::pixmap(const QSize &size, QIcon::Mode mode,
             if (d->svgBuffers && !d->svgBuffers->isEmpty())
                 mdst = QByteArray::number(d->hashKey(mode, state)).prepend("_");
             // svgFiles[-1] 是为了取到额外的cache key,例如会跟随颜色变化的svg图标，将会添加此特殊的标记
-            const QByteArray cacheKey = svgFile.toLocal8Bit() + d->svgFiles[-1].toLocal8Bit() + mdst; // add mode state to cachekey
+            const QByteArray extraKey = d->svgFiles.value(-1).isEmpty() ? QByteArrayLiteral("") : d->svgFiles.value(-1).toLocal8Bit() + mdst;
+            const QByteArray cacheKey = svgFile.toLocal8Bit() + extraKey; // add mode state to cachekey
             const QString &svgFileSha1 = QString::fromLatin1(QCryptographicHash::hash(cacheKey, QCryptographicHash::Sha1).toHex());
             cacheFile = QStringLiteral("%1/%2.png").arg(cachePath).arg(svgFileSha1);
         }
@@ -364,10 +365,15 @@ void QSvgIconEngine::addFile(const QString &fileName, const QSize &,
 void QSvgIconEngine::paint(QPainter *painter, const QRect &rect,
                            QIcon::Mode mode, QIcon::State state)
 {
-    QSize pixmapSize = rect.size();
-    if (painter->device())
-        pixmapSize *= painter->device()->devicePixelRatioF();
-    painter->drawPixmap(rect, pixmap(pixmapSize, mode, state));
+    qreal ratio = 1.0;
+    if (qApp->testAttribute(Qt::AA_UseHighDpiPixmaps))
+        ratio = painter->device() ? painter->device()->devicePixelRatioF() : qApp->devicePixelRatio();
+
+    QSize pixmapSize = rect.size() * ratio;
+    QPixmap pix = pixmap(pixmapSize, mode, state);
+
+    pix.setDevicePixelRatio(ratio);
+    painter->drawPixmap(rect, pix);
 }
 
 QString QSvgIconEngine::key() const
